@@ -308,10 +308,11 @@ def __manpages(attrs, src):
     for section, files in mans.items():
         v.append((os.path.join('share', 'man', 'man' + section), files))
 
-def __external_mod(module, attrs):
+def __external_mod(cur_module, module, attrs):
     '''Check if given Python module is not included in Python or locally'''
 
-    # filter out locally provided modules
+    # filter out locally provided modules early, to avoid importing them (which
+    # might raise an exception, or parse argv, etc.)
     if module in attrs['provides']:
         return False
     for m in _module_parents(module):
@@ -319,33 +320,55 @@ def __external_mod(module, attrs):
             return False
 
     try:
-        path = __import__(module).__file__
+        mod = __import__(module)
     except ImportError:
-        sys.stderr.write('ERROR: Python module %s not found\n' % module)
-        return False
-    except AttributeError: # builtin modules
+        # try relative import
+        try:
+            if cur_module:
+                mod = __import__(cur_module + '.' + module)
+            else:
+                raise ImportError
+        except ImportError:
+            sys.stderr.write('ERROR: Python module %s not found\n' % module)
+            return False
+    except ValueError: # weird ctypes case with wintypes
+        return False 
+
+    if not hasattr(mod, '__file__'):
+        # builtin module
         return False
 
-    return 'dist-packages' in path or 'site-packages' in path or \
-            not path.startswith(os.path.dirname(os.__file__))
+    # filter out locally provided modules
+    if mod.__name__ in attrs['provides']:
+        return False
+
+    return 'dist-packages' in mod.__file__ or 'site-packages' in mod.__file__ or \
+            not mod.__file__.startswith(os.path.dirname(os.__file__))
 
 def __add_imports(imports, file, attrs):
     '''Add all imported modules from file to imports set.
 
     This filters out modules which are shipped with Python itself.
     '''
+    if os.path.exists(os.path.join(os.path.dirname(file), '__init__.py')):
+        cur_module = '.'.join(file.split(os.path.sep)[:-1])
+    else:
+        # this might happen for paths like bin/<script> which we do not want to
+        # treat as module for checking relative imports
+        cur_module = None
+
     try:
         tree = ast.parse(open(file).read(), file)
 
         for node in ast.walk(tree):
             if isinstance(node, ast.Import):
                 for alias in node.names:
-                    if __external_mod(alias.name, attrs):
+                    if __external_mod(cur_module, alias.name, attrs):
                         imports.add(alias.name)
             if isinstance(node, ast.ImportFrom):
-                if __external_mod(node.module, attrs):
+                if __external_mod(cur_module, node.module, attrs):
                     imports.add(node.module)
-    except SyntaxError, e:
+    except SyntaxError as e:
         sys.stderr.write('WARNING: syntax errors in %s: %s\n' % (file, str(e)))
 
 def _module_parents(mod):
@@ -659,26 +682,29 @@ class install_auto(distutils.command.install.install):
                     preserve_times=0, preserve_symlinks=1, verbose=1)
 
         # install data/scripts symlinks
-        for f in distutils.filelist.findall():
-            if not os.path.islink(f):
-                continue
-            if f.startswith('bin/') or f.startswith('data/'):
-                if f.startswith('bin'):
-                    dir = self.install_scripts
-                    dest = os.path.join(dir, os.path.sep.join(f.split(os.path.sep)[1:]))
-                elif f.startswith('data/icons'):
-                    dir = os.path.join(self.install_data, 'share', 'icons', 'hicolor')
-                    dest = os.path.join(dir, os.path.sep.join(f.split(os.path.sep)[2:]))
-                else:
-                    dir = os.path.join(self.install_data, 'share', self.distribution.get_name())
-                    dest = os.path.join(dir, os.path.sep.join(f.split(os.path.sep)[1:]))
+        for (path, dirs, files) in os.walk('.'):
+            for f in files:
+                f = os.path.join(path, f)
+                if not os.path.islink(f):
+                    continue
 
-                d = os.path.dirname(dest)
-                if not os.path.isdir(d):
-                    os.makedirs(d)
-                if os.path.exists(dest):
-                    os.unlink(dest)
-                os.symlink(os.readlink(f), dest)
+                if f.startswith('./bin/') or f.startswith('./data/'):
+                    if f.startswith('./bin'):
+                        dir = self.install_scripts
+                        dest = os.path.join(dir, os.path.sep.join(f.split(os.path.sep)[2:]))
+                    elif f.startswith('./data/icons'):
+                        dir = os.path.join(self.install_data, 'share', 'icons', 'hicolor')
+                        dest = os.path.join(dir, os.path.sep.join(f.split(os.path.sep)[3:]))
+                    else:
+                        dir = os.path.join(self.install_data, 'share', self.distribution.get_name())
+                        dest = os.path.join(dir, os.path.sep.join(f.split(os.path.sep)[2:]))
+
+                    d = os.path.dirname(dest)
+                    if not os.path.isdir(d):
+                        os.makedirs(d)
+                    if os.path.exists(dest):
+                        os.unlink(dest)
+                    os.symlink(os.readlink(f), dest)
 
         distutils.command.install.install.run(self)
 
