@@ -290,10 +290,14 @@ Exec=/bin/foosettings''')
 
         self._mksrc('data/icons/scalable/actions/press.png')
         self._mksrc('data/icons/48x48/apps/foo.png')
-        action_icon_path = os.path.join(self.src, 'data', 'icons', 'scalable',
-                'actions')
-        os.symlink(os.path.join(action_icon_path, 'press.png'),
-                os.path.join(action_icon_path, 'crunch.png'))
+        scalable_icon_path = os.path.join(self.src, 'data', 'icons', 'scalable')
+        os.symlink(os.path.join(scalable_icon_path, 'actions', 'press.png'),
+                os.path.join(scalable_icon_path, 'actions', 'crunch.png'))
+
+        # test broken symlink, too
+        os.mkdir(os.path.join(scalable_icon_path, 'mimetypes'))
+        os.symlink('../apps/foo.svg', 
+                os.path.join(scalable_icon_path, 'mimetypes', 'text-x-foo.svg'))
 
         (o, e, s) = self.do_install()
         self.assertEqual(e, '')
@@ -306,6 +310,8 @@ Exec=/bin/foosettings''')
         self.assertTrue('/usr/share/icons/hicolor/48x48/apps/foo.png' in f)
         self.assertTrue(os.path.islink(os.path.join(self.install_tree, 
            'usr/share/icons/hicolor/scalable/actions/crunch.png')))
+        self.assertTrue(os.path.islink(os.path.join(self.install_tree, 
+           'usr/share/icons/hicolor/scalable/mimetypes/text-x-foo.svg')))
 
     def test_data(self):
         '''Auxiliary files in data/'''
@@ -671,12 +677,20 @@ setup(
 import os, os.path, email.mime, distutils.command.register
 from email import header as h
 import httplib2.iri2uri, unknown
+from . bar import poke
+from bar.poke import x
+import grab_cli
+import broken
 ''')
 
         self._mksrc('foo/bar/__init__.py', '')
         self._mksrc('foo/bar/poke.py', 'def x(): pass')
 
         self._mksrc('mymod.py', 'import foo\nfrom foo.bar.poke import x')
+        # trying to import this will cause setup.py to not process any args any more
+        self._mksrc('grab_cli.py', 'from optparse import OptionParser\nOptionParser().parse_args()')
+        # trying to import this will break setup.py
+        self._mksrc('broken.py', 'raise SystemError("cannot initialize system")')
 
         self._mksrc('bin/foo-cli', '''#!/usr/bin/python
 import sys
@@ -693,7 +707,7 @@ print ('import iamnota.module')
         self._mksrc('data/example-code/mymod/shiny.py', 'import example.othermod')
 
         (o, e, s) = self.do_install()
-        self.assertEqual(s, 0)
+        self.assertEqual(s, 0, e)
         self.assertEqual(e, 'ERROR: Python module unknown not found\n')
         self.assertFalse('following files are not recognized' in o)
 
@@ -712,7 +726,7 @@ print ('import iamnota.module')
 
         # check provides
         prov = [prop.split(' ', 1)[1] for prop in egg if prop.startswith('Provides: ')]
-        self.assertEqual(set(prov), set(['foo', 'mymod']))
+        self.assertEqual(set(prov), set(['foo', 'mymod', 'broken', 'grab_cli']))
 
         # check requires
         req = [prop.split(' ', 1)[1] for prop in egg if prop.startswith('Requires: ')]
@@ -856,7 +870,7 @@ print ('import iamnota.module')
         assert self.snapshot is None, 'snapshot already taken'
 
         self.snapshot = tempfile.mkdtemp()
-        shutil.copytree(self.src, os.path.join(self.snapshot, 's'))
+        shutil.copytree(self.src, os.path.join(self.snapshot, 's'), symlinks=True)
 
     def diff_snapshot(self):
         '''Compare source tree to snapshot.
@@ -864,11 +878,11 @@ print ('import iamnota.module')
         Return diff -Nur output.
         '''
         assert self.snapshot, 'no snapshot taken'
-        diff = subprocess.Popen(['diff', '-x', 'foo.pot', '-Nur', os.path.join(self.snapshot, 's'), 
-            self.src], stdout=subprocess.PIPE, stderr=subprocess.PIPE)
+        diff = subprocess.Popen(['diff', '-x', 'foo.pot', '-x', '*.pyc',
+            '-Nur', os.path.join(self.snapshot, 's'), self.src],
+            stdout=subprocess.PIPE, stderr=subprocess.PIPE)
         (out, err) = diff.communicate()
-        out = out.decode()
-        self.assertEqual(err, b'', 'diff error messages')
+        out = out.decode('UTF-8')
         return out
 
     def _mkpo(self):
